#! /usr/bin/env python

"""Solve a few STRIPS tasks with A* and the LM-Cut heuristic. Once with Translation done with Helmerts Algorithm from 2009 and once done with Rintans Algorithm from 2008"""

import os
import os.path
import platform

from downward.experiment import FastDownwardExperiment
from downward.reports.absolute import AbsoluteReport
from downward.reports.scatter import ScatterPlotReport
from lab import cached_revision
from lab.environments import BaselSlurmEnvironment, LocalEnvironment


ATTRIBUTES = ["coverage", "cost", "evaluations", "planner_time","search_time","total_time","initial_h_value", "error"]

NODE = platform.node()
if NODE.endswith(".scicore.unibas.ch") or NODE.endswith(".cluster.bc2.ch"):
    # Create bigger suites with suites.py from the downward-benchmarks repo.
    SUITE = ["depot:p01.pddl", "pipesworld-notankage:p01-net1-b6-g2.pddl",  "satellite:p01-pfile1.pddl"]
    ENV = BaselSlurmEnvironment(email="severin.wyss@stud.unibas.ch")
else:
    SUITE = ["depot:p01.pddl", "pipesworld-notankage:p01-net1-b6-g2.pddl",  "satellite:p01-pfile1.pddl"]
    ENV = LocalEnvironment(processes=2)
# Use path to your Fast Downward repository.
REPO = "MT1/Code"
BENCHMARKS_DIR = "MT1/benchmark"
# If REVISION_CACHE is None, the default ./data/revision-cache is used.
REVISION_CACHE = os.environ.get("DOWNWARD_REVISION_CACHE")
REV = "sw_mt_invarsynt_main"

exp = FastDownwardExperiment(environment=ENV, revision_cache=REVISION_CACHE)

# Add built-in parsers to the experiment.
exp.add_parser(exp.EXITCODE_PARSER)
exp.add_parser(exp.TRANSLATOR_PARSER)
exp.add_parser(exp.SINGLE_SEARCH_PARSER)
exp.add_parser(exp.PLANNER_PARSER)

exp.add_suite(BENCHMARKS_DIR, SUITE)

exp.add_algorithm("lmcut_H", REPO, REV, ["--search", "astar(lmcut())"])
exp.add_algorithm("lmcut_R", REPO, REV, ["--search", "astar(lmcut())", "--translate-options", "--rintan08", "--rintan08_n", "2"],[],[ "--translate-time-limit", "5m"])

# Add step that writes experiment files to disk.
exp.add_step("build", exp.build)

# Add step that executes all runs.
exp.add_step("start", exp.start_runs)

# Add step that collects properties from run directories and
# writes them to *-eval/properties.
exp.add_fetcher(name="fetch")

# Add report step (AbsoluteReport is the standard report).
exp.add_report(AbsoluteReport(attributes=ATTRIBUTES), outfile="report.html")

# Add scatter plot report step.
exp.add_report(
    ScatterPlotReport(attributes=["expansions"], filter_algorithm=["lmcut"]),
    outfile="scatterplot.png",
)

# Parse the commandline and show or run experiment steps.
exp.run_steps()
